#pragma ident "$Id: //depot/sgl/gpstk/dev/apps/reszilla/aoa2rin.cpp#1 $"

// aoa2rin: translate AOA's development output into RINEX obs files


#include "util.hpp"

using namespace std;

int verbosity=0;

int main(int argc, char *argv[])
{
   try {
      gpstk::CommandOptionNoArg
         helpOption('h', "help", "Print help usage");

      gpstk::CommandOptionWithAnyArg
         inputFileOption('i',"input-file", "Input file name", true),
         outputFileOption('o',"output-file", "Output file name", true);

      gpstk::CommandOptionWithNumberArg
         rxNumOption('n', "rx-number", "The receiver number", true),
         debugOption('d', "debug", "Increase debug level");

      string appDesc("This program translates the development output of the AOA MSN receiver and writes it in RINEX format");
      gpstk::CommandOptionParser cop(appDesc);
      cop.parseOptions(argc, argv);

      if (helpOption.getCount() || cop.hasErrors())
      {
         if (cop.hasErrors())
            cop.dumpErrors(cout);
         cop.displayUsage(cout);
         exit(0);
      }

      if (debugOption.getCount())
         verbosity = gpstk::StringUtils::asInt(debugOption.getValue()[0]);

      gpstk::RinexObsStream ros;
      std::ifstream input;
      std::string ifn,ofn;
      
      ifn = (inputFileOption.getValue())[0];
      ofn = (outputFileOption.getValue())[0];

      ros.open(ofn.c_str(), std::ios::out);
      
      if (verbosity)
      {
         std::cout << "Debug Level: " << verbosity << std::endl;
         std::cout << "Reading from " << ifn << std::endl;
         std::cout << "Writing to " << ofn << std::endl;
      }

      gpstk::RinexObsHeader roh;
      RODEpochMap rem;
      read_aoa_data(ifn, rem, roh);
      roh.antennaPosition[0] = -2547874.7941;
      roh.antennaPosition[1] = -4628781.4123;
      roh.antennaPosition[2] = 3561044.0127;

      ros << roh;
      RODEpochMap::const_iterator i;
      for (i=rem.begin(); i!=rem.end(); i++)
         ros << i->second;
   }
   catch (gpstk::Exception& e)
   {
      cout << endl << "Caught an excption." << endl << e << endl
           << "Terminating." << endl;
   }
} // main()
