#pragma ident "$Id: //depot/sgl/gpstk/dev/apps/reszilla/util.hpp#4 $"

#include <iostream>
#include <iomanip>
#include <string>
#include <map>
#include <set>

// gpstk crud
#include "StringUtils.hpp"
#include "DayTime.hpp"
#include "stl_helpers.hpp"


#include "EphemerisStore.hpp"
#include "icd_200_constants.hpp"
#include "GPSGeoid.hpp"
#include "TropModel.hpp"
#include "Geodetic.hpp"

#include "Stats.hpp"

//Local Crud
#include "ORDEpoch.hpp"
#include "EpochClockModel.hpp"
#include "LinearClockModel.hpp"
#include "readers.hpp"


extern int verbosity;

// a store of ResidualEpoch objects
typedef std::map<gpstk::DayTime, gpstk::ORDEpoch> ORDEpochMap;

typedef gpstk::RinexObsHeader::RinexObsType RinexObsType;

extern const RinexObsType& C1;
extern const RinexObsType& D1;
extern const RinexObsType& D2;
extern const RinexObsType& L1;
extern const RinexObsType& L2;
extern const RinexObsType& P1;
extern const RinexObsType& P2;

extern std::string timeFormat;

void compute_ords(ORDEpochMap& oem,
                  const RODEpochMap& rem,
                  const gpstk::RinexObsHeader& roh,
                  const gpstk::EphemerisStore& bce,
                  const gpstk::WxObsData& wod,
                  bool svTime,
                  const std::string& ordModeStr,
                  const std::string& clkModelStr);

void dump(std::ostream& s, const ORDEpochMap& oem);
void dumpStats(const ORDEpochMap& oem, 
               const std::string& ordMode,
               const double sigmam=5);
void computeStats(const std::string desc,
                  const ORDEpochMap& oem,
                  const double minElevation,
                  const double maxElevation, 
                  const double sigmam);


// The key for this map is the prn of the SV
//  PrnElevationMap pem;
//  pem[time][prn] = elevation;
typedef std::map<gpstk::DayTime, std::map<short, float> > PrnElevationMap;
PrnElevationMap elevation_map(const ORDEpochMap& oem);
PrnElevationMap elevation_map(const RODEpochMap& rem,
                  const gpstk::RinexObsHeader& roh,
                  const gpstk::EphemerisStore& bce);

struct DDResidual
{
   typedef std::map<RinexObsType, double> DDD;
   // the receiver residuals
   DDD res;
   // The phase offset (only for phase observables)
   DDD bias;

   friend std::ostream& operator<<(std::ostream& s, 
                                   const DDResidual& r) throw();
};

struct DDEpoch
{
   typedef std::map<gpstk::RinexPrn, DDResidual> DDResidualMap;
   DDResidualMap res;
   double clockOffset;
   gpstk::RinexPrn masterPrn;

   // Returns true if successfull
   bool compute(gpstk::RinexObsData rx1,
                gpstk::RinexObsData rx2,
                gpstk::RinexPrn masterPrn);
};
typedef std::map<gpstk::DayTime, DDEpoch> DDEpochMap;
void dump(std::ostream& s,
          DDEpochMap& ddem,
          const RinexObsType rot,
          PrnElevationMap& pem);
void dumpStats(DDEpochMap& oem, PrnElevationMap& pem);
void computeStats(const RinexObsType rot,
                  DDEpochMap& oem,
                  const double minElevation,
                  const double maxElevation,
                  PrnElevationMap& pem);
void computeDD(const RODEpochMap& rx1,
               const RODEpochMap& rx2,
               double minMasterElevation,
               DDEpochMap& ddem,
               PrnElevationMap& pem);



struct DD2Epoch
{
   typedef std::map<gpstk::RinexPrn, double> DD2ResidualMap;
   DD2ResidualMap res;

   // Returns true if successfull
   bool compute(gpstk::RinexObsData rx1,
                gpstk::RinexObsData rx2);
};
typedef std::map<gpstk::DayTime, DD2Epoch> DD2EpochMap;
void dump(std::ostream& s,
          DD2EpochMap& ddem, 
          PrnElevationMap& pem);
void dumpStats(DD2EpochMap& ddem, PrnElevationMap& pem);
void computeStats(DD2EpochMap& ddem,
                  const double minElevation,
                  const double maxElevation, 
                  PrnElevationMap& pem);
void computeDD(const RODEpochMap& rx1,
               const RODEpochMap& rx2,
               DD2EpochMap& ddem);




struct CycleSlipRecord
{
   gpstk::DayTime t;
   double cycles;
   gpstk::RinexPrn  prn, masterPrn;
};

typedef std::list<CycleSlipRecord> CycleSlipList;


void add_clock_to_rinex(RODEpochMap& rem, const ORDEpochMap& oem);


CycleSlipList debiasPhase(const RinexObsType rot, DDEpochMap& ddem);
void dumpStats(const CycleSlipList& sl, const RinexObsType rot, PrnElevationMap& pem);
